import React, {Component} from 'react'
import PropTypes from 'prop-types'
import { is, fromJS } from 'immutable'
import { connect } from 'react-redux'
import { Spin, notification, Button } from 'antd'

import Api from '@/api'
import UtilsDM from '@/utils/utils-datamanage.js'
import asyncComponent from '@/utils/asyncComponent'
import asyncSpinComponent from '@/utils/asyncSpinComponent'
import MKEmitter from '@/utils/events.js'
import zhCN from '@/locales/zh-CN/main.js'
import enUS from '@/locales/en-US/main.js'
import './index.scss'

const MutilForm = asyncSpinComponent(() => import('@/tabviews/zshare/mutilform'))
const NormalButton = asyncComponent(() => import('@/tabviews/zshare/actionList/normalbutton'))

class StepForm extends Component {
  static propTpyes = {
    data: PropTypes.array,           // 统一查询数据
    config: PropTypes.object,        // 组件配置信息
    mainSearch: PropTypes.any,       // 外层搜索条件
  }

  state = {
    dict: sessionStorage.getItem('lang') !== 'en-US' ? zhCN : enUS,
    BID: '',
    config: null,
    loading: false,
    sync: false,
    data: null,
    group: null,
    BData: '',
    step: 0
  }

  UNSAFE_componentWillMount () {
    const { data } = this.props
    let config = fromJS(this.props.config).toJS()

    let _data = null
    let _sync = false

    let BID = ''
    let BData = ''

    if (config.setting.supModule) {
      BData = window.GLOB.CacheData.get(config.setting.supModule)
    } else {
      BData = window.GLOB.CacheData.get(config.$pageId)
    }
    if (BData) {
      BID = BData.$BID || ''
    }

    if (config.wrap.datatype !== 'static') {
      _sync = config.setting.sync === 'true'

      if (_sync && data && data[config.dataName]) {
        _data = data[config.dataName]
        if (Array.isArray(_data)) {
          _data = _data[0] || {$$empty: true}
        }
        _sync = false
      }
    } else {
      _data = {$$empty: true}
    }

    if (!config.wrap.groupLabel) {
      if (config.subcards.length > 1) {
        config.wrap.groupLabel = 'show'
      } else {
        config.wrap.groupLabel = 'hidden'
      }
    }

    config.subcards = config.subcards.map((group, i) => {
      group.sort = i + 1
      if (i === 0) {
        group.prevButton.enable = 'false'
      }
      if (i + 1 === config.subcards.length) {
        group.nextButton.enable = 'false'
      }

      if (group.subButton.enable === 'false') {
        group.subButton.style.display = 'none'
      }

      if (group.prevButton.enable === 'false' && group.subButton.enable === 'false' && group.nextButton.enable === 'false') {
        group.$button = 'no-button'
      }

      return group
    })

    let _group = config.subcards[0]

    if (_data && config.wrap.statusControl && _data[config.wrap.statusControl]) {
      let _status = _data[config.wrap.statusControl]

      let _groups = config.subcards.filter(item => item.setting.status === _status)[0]
      _group = _groups || _group
    }

    this.setState({
      sync: _sync,
      data: _data,
      group: _group,
      step: _group.sort - 1,
      BID: BID || '',
      BData: BData || '',
      config: config,
      arr_field: config.columns.map(col => col.field).join(','),
    }, () => {
      if (config.wrap.datatype !== 'static' && config.setting && config.setting.sync !== 'true' && config.setting.onload === 'true') {
        setTimeout(() => {
          this.loadData()
        }, config.setting.delay || 0)
      }
    })
  }

  componentDidMount () {
    MKEmitter.addListener('reloadData', this.reloadData)
    MKEmitter.addListener('mkFormSubmit', this.mkFormSubmit)
    MKEmitter.addListener('resetSelectLine', this.resetParentParam)
    MKEmitter.addListener('refreshByButtonResult', this.refreshByButtonResult)
  }

  shouldComponentUpdate (nextProps, nextState) {
    return !is(fromJS(this.state), fromJS(nextState))
  }

  componentWillUnmount () {
    this.setState = () => {
      return
    }
    MKEmitter.removeListener('reloadData', this.reloadData)
    MKEmitter.removeListener('mkFormSubmit', this.mkFormSubmit)
    MKEmitter.removeListener('resetSelectLine', this.resetParentParam)
    MKEmitter.removeListener('refreshByButtonResult', this.refreshByButtonResult)
  }

  /**
   * @description 图表数据更新，刷新内容
   */
  UNSAFE_componentWillReceiveProps (nextProps) {
    const { sync, config, group } = this.state

    if (sync && !is(fromJS(this.props.data), fromJS(nextProps.data))) {
      let _data = {$$empty: true}
      let _group = group
      if (nextProps.data && nextProps.data[config.dataName]) {
        _data = nextProps.data[config.dataName]
        if (Array.isArray(_data)) {
          _data = _data[0] || {$$empty: true}
        }
      }

      if (config.wrap.statusControl && _data[config.wrap.statusControl]) {
        let _status = _data[config.wrap.statusControl]
        let _groups = config.subcards.filter(item => item.setting.status === _status)[0]
        _group = _groups || _group
      }

      this.setState({sync: false, data: _data, group: _group, step: _group.sort - 1,})
    } else if (config.setting.syncRefresh && nextProps.mainSearch && !is(fromJS(this.props.mainSearch), fromJS(nextProps.mainSearch))) {
      this.setState({}, () => {
        this.loadData()
      })
    }
  }

  reloadData = (menuId, id) => {
    const { config } = this.state

    if (config.uuid !== menuId) return

    this.loadData('refresh')
  }

  /**
   * @description 按钮执行完成后页面刷新
   * @param {*} menuId     // 菜单Id
   * @param {*} position   // 刷新位置
   * @param {*} btn        // 执行的按钮
   */
  refreshByButtonResult = (menuId, position, btn, id) => {
    const { config, group, BID } = this.state

    if (group.uuid !== menuId) return

    if (position === 'mainline' && config.setting.supModule && BID) {
      MKEmitter.emit('reloadData', config.setting.supModule, BID)
    } else {
      this.loadData()
    }

    if (id) {
      MKEmitter.emit('resetSelectLine', config.uuid, id, '')
    }

    this.execSuccess(btn, id)
  }

  resetParentParam = (MenuID, id, data) => {
    const { config } = this.state

    if (!config.setting.supModule || config.setting.supModule !== MenuID) return
    
    if (id !== this.state.BID || id !== '') {
      if (config.wrap.datatype === 'static' || (config.setting.supModule && !id)) {
        this.setState({
          data: null,
          BID: id,
          BData: data
        }, () => {
          this.setState({
            data: {$$empty: true}
          })
        })
      } else {
        this.setState({ BID: id, BData: data }, () => {
          this.loadData()
        })
      }
    }
  }

  execSuccess = (btn, id) => {
    const { config, group } = this.state

    if (config.subcards.length > group.sort) {
      let _group = config.subcards.filter(item => item.sort === (group.sort + 1))[0]
  
      this.setState({group: null, step: group.sort}, () => {
        this.setState({group: _group})
      })
    } else {
      this.setState({step: group.sort})
    }

    if (btn.linkmenu && btn.linkmenu.length > 0) {
      let menu_id = btn.linkmenu[btn.linkmenu.length - 1]
      let menu = this.props.permMenus.filter(m => m.MenuID === menu_id)[0] || ''

      if (!menu) return

      let newtab = {
        ...menu,
        param: {$BID: id || ''}
      }

      if (['linkage_navigation', 'linkage', 'menu_board'].includes(window.GLOB.navBar)) {
        MKEmitter.emit('modifyTabs', newtab, 'replace')
      } else {
        MKEmitter.emit('modifyTabs', newtab, 'plus', true)
      }
    }
  }

  async loadData (type) {
    const { mainSearch } = this.props
    const { config, arr_field, BID } = this.state

    if (config.wrap.datatype === 'static') {
      this.setState({
        data: {$$empty: true}
      })
      return
    } else if (config.setting.supModule && !BID) {
      this.setState({
        data: null
      }, () => {
        this.setState({data: {$$empty: true}})
      })
      return
    }

    let searches = config.setting.useMSearch && mainSearch ? mainSearch : []

    let requireFields = searches.filter(item => item.required && item.value === '')
    if (requireFields.length > 0) {
      return
    }

    this.setState({
      loading: true
    })

    let _orderBy = config.setting.order || ''
    let param = UtilsDM.getQueryDataParams(config.setting, arr_field, searches, _orderBy, 1, 1, BID)

    let result = await Api.genericInterface(param)
    if (result.status) {
      let _data = result.data && result.data[0] ? result.data[0] : {$$empty: true}
      let _group = this.state.group

      if (type === 'refresh') {
        _group = config.subcards[0]
      }

      if (config.wrap.statusControl && _data[config.wrap.statusControl]) {
        let _status = _data[config.wrap.statusControl]
        let _groups = config.subcards.filter(item => item.setting.status === _status)[0]
        _group = _groups || _group
      }

      let step = this.state.step

      if (config.subcards.length !== _group.sort || config.subcards.length !== step) {
        step = _group.sort - 1
      }

      this.setState({
        group: null,
        step: step,
        data: _data,
        loading: false
      }, () => {
        this.setState({group: _group})
      })
    } else {
      this.setState({
        loading: false,
      })
      notification.error({
        top: 92,
        message: result.message,
        duration: 10
      })
    }
  }

  mkFormSubmit = (btnId) => {
    const { group } = this.state

    if (group.uuid !== btnId) return

    this.formRef.handleConfirm().then(res => {
      MKEmitter.emit('triggerFormSubmit', {menuId: btnId, form: res})
    })
  }

  prevStep = () => {
    const { config, group } = this.state

    let _group = config.subcards.filter(item => item.sort === (group.sort - 1))[0]

    this.setState({group: null, step: group.sort - 2}, () => {
      this.setState({group: _group})
    })
  }

  nextStep = () => {
    const { config, group } = this.state

    let _group = config.subcards.filter(item => item.sort === (group.sort + 1))[0]

    this.setState({group: null, step: group.sort}, () => {
      this.setState({group: _group})
    })
  }

  render() {
    const { config, loading, BID, BData, data, group, dict, step } = this.state

    return (
      <div className="custom-normal-form-box" id={'anchor' + config.uuid} style={{...config.style}}>
        {loading ?
          <div className="loading-mask">
            <div className="ant-spin-blur"></div>
            <Spin />
          </div> : null
        }
        {config.wrap.groupLabel !== 'hidden' ? <div className="mk-normal-form-title">
          {config.subcards.map(card => (
            <div key={card.uuid} className={'form-title' + (card.sort <= step ? ' active' : '')}>
              <span className="form-sort" style={{background: config.wrap.color}}>{card.sort}</span>
              <span className="before-line" style={{background: config.wrap.color}}></span>
              <span className="after-line" style={{background: config.wrap.color}}></span>
              {card.setting.title}
            </div>))
          }
        </div> : null}
        {group && data ? <MutilForm
          BID={BID}
          BData={BData}
          dict={dict}
          data={data}
          action={group}
          inputSubmit={() => this.mkFormSubmit(group.uuid)}
          wrappedComponentRef={(inst) => this.formRef = inst}
        /> : null}
        {group && data ? <div className={'mk-form-action ' + (group.$button || '')}>
          {group.prevButton.enable === 'true' ? <Button type="link" className="prev" onClick={this.prevStep} style={group.prevButton.style}>{group.prevButton.label}</Button> : null}
          <NormalButton
            BID={BID}
            btn={group.subButton}
            setting={config.setting}
            columns={config.columns}
            selectedData={data.$$empty ? [] : [data]}
          />
          {group.nextButton.enable === 'true' ? <Button type="link" className="skip" onClick={this.nextStep} style={group.nextButton.style}>{group.nextButton.label}</Button> : null}
        </div> : null}
      </div>
    )
  }
}

const mapStateToProps = (state) => {
  return {
    permMenus: state.permMenus,
  }
}

const mapDispatchToProps = () => {
  return {}
}

export default connect(mapStateToProps, mapDispatchToProps)(StepForm)